#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

typedef enum {
	END_AT = 0, END_AFTER, CANCEL_COUNTDOWN, ACTION_SIZE
} ACTION;

char *action_string[] = {
	"=", "+", "+0:0:0", NULL
};

static ACTION action = END_AT;
extern GtkWidget *main_dialog;


void
on_main_dialog_destroy                 (GtkObject       *object,
                                        gpointer         user_data)
{
	gtk_main_quit();
}


void
on_close_button_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_main_quit();
}


void
on_ok_button_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *fifo_entry = lookup_widget(main_dialog, "fifo_entry");
	GtkWidget *hours = lookup_widget(main_dialog, "hours_spinbutton");
	GtkWidget *mins = lookup_widget(main_dialog, "minutes_spinbutton");
	GtkWidget *secs = lookup_widget(main_dialog, "seconds_spinbutton");
	gchar fifo_name[128];
	gchar buf[256];
	FILE *save, *fifo;
	int fd;
	
	strcpy(fifo_name, gtk_entry_get_text(GTK_ENTRY(fifo_entry)));

	/* save configuration (name of wmcdclock's fifo) */
	strcpy(buf, getenv("HOME"));
	strcat(buf, "/.gwmcdclock-set.cfg");
	if((save = fopen(buf, "w")) != NULL) {
		fprintf(save, "%s", fifo_name);
		fclose(save);
	}
	
	/* s/^~/$HOME/ */
	if (fifo_name[0] == '~') {
		strncpy(buf, getenv("HOME"), 128);
		strcat(buf, fifo_name + sizeof(char));
		strcpy(fifo_name, buf);
	}	
	
	/* prepare command */
	if (action != CANCEL_COUNTDOWN) {
		sprintf(buf, "%s%02d:%02d:%02d", action_string[action],
			(int)gtk_spin_button_get_value(GTK_SPIN_BUTTON(hours)),
			(int)gtk_spin_button_get_value(GTK_SPIN_BUTTON(mins)),
			(int)gtk_spin_button_get_value(GTK_SPIN_BUTTON(secs)));
	} else {
		strcpy(buf, action_string[action]);
	}
	
	/* do action */
	if((fd = open(fifo_name, O_WRONLY)) > 0 
			&& (fifo = fdopen(fd, "w")) != NULL) {
		fprintf(fifo, "%s\n", buf);
		fclose(fifo);
		
		/* konec */
		gtk_main_quit();
	} else {
		perror(fifo_name);
	}
}


void
on_end_after_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *time = lookup_widget(main_dialog, "time_hbox");
	gtk_widget_set_sensitive(time, TRUE);
	action = END_AFTER;
}


void
on_end_at_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *time = lookup_widget(main_dialog, "time_hbox");
	gtk_widget_set_sensitive(time, TRUE);
	action = END_AT;
}


void
on_cancel_countdown_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *time = lookup_widget(main_dialog, "time_hbox");
	gtk_widget_set_sensitive(time, FALSE);
	action = CANCEL_COUNTDOWN;
}

